#include <windows.h>
#include <GL/glut.h>
#include <math.h>

#define M_PI 3.14159265358979323846
#define CORNERS 256
#define INTERVAL 33

struct Ball
{
	double x, y;
	double vx, vy;
	double radius;
	double r, g, b;
};

Ball ball;

void reshape(int w, int h)
{
	glViewport(0,0,w,h);

	if (w == 0) w = 1;
	if (h == 0) h = 1;

	double aspect = (double)w / h;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (w > h)
		gluOrtho2D(-aspect, aspect, -1, 1);
	else
		gluOrtho2D(-1, 1, -1/aspect, 1/aspect);
}

void drawBall(Ball &b)
{
	glColor3d(b.r, b.g, b.b);
	glBegin(GL_TRIANGLE_FAN);
	{
		glVertex2d(b.x, b.y);
		for(int i=0;i<CORNERS;i++)
		{
			double theta = 2*M_PI*i/CORNERS;
			double x = b.x + b.radius * cos(theta);
			double y = b.y + b.radius * sin(theta);
			glVertex2d(x,y);
		}
		glVertex2d(b.x + b.radius, b.y);
	}
	glEnd();
}

void display()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	
	drawBall(ball);

	glFlush();
}

void updateBall(Ball &b)
{
	b.x += b.vx;
	b.y += b.vy;

	if (b.x + b.radius > 1)
	{
		b.x = 1 - b.radius;
		b.vx = -fabs(b.vx);
	}
	if (b.x - b.radius < -1)
	{
		b.x = -1 + b.radius;
		b.vx = fabs(ball.vx);
	}
	if (b.y + b.radius > 1)
	{
		b.y = 1 - ball.radius;
		b.vy = -fabs(b.vy);
	}
	if (b.y - b.radius < -1)
	{
		b.y = -1 + b.radius;
		b.vy = fabs(b.vy);
	}
}

void animate(int param)
{
	updateBall(ball);
	glutTimerFunc(INTERVAL, animate, 0);
	glutPostRedisplay();
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_SINGLE);
	glutInitWindowSize(600,600);
	glutCreateWindow("Bouncing Ball");
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
	glutTimerFunc(0, animate, 0);

	ball.x = 0;
	ball.y = 0;
	ball.vx = 0.02;
	ball.vy = 0.03;
	ball.radius = 0.1;
	ball.r = 1.0;
	ball.g = 0.5;
	ball.b = 0.5;

	glutMainLoop();
}