#include <string.h>
#include <GL/glui.h>

int mainWindow;

GLenum frontFaceMode = GL_CCW;
GLenum cullFaceMode = GL_BACK;
int cullingEnabled = 0;

int cullFaceChoice = 0;
int frontFaceChoice = 0;

void idle()
{
	if (glutGetWindow() != mainWindow)
		glutSetWindow(mainWindow);
	glutPostRedisplay();
}

void reshape(int w, int h)
{
	glViewport(0,0,w,h);
	
	if (w == 0) w = 1;
	if (h == 0) h = 1;

	double aspect = w * 1.0 / h;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (w > h)
		gluOrtho2D(-aspect, aspect, -1, 1);
	else
		gluOrtho2D(-1, 1, -1/aspect, 1/aspect);
}


void display()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);

	if (cullingEnabled)
		glEnable(GL_CULL_FACE);
	else
		glDisable(GL_CULL_FACE);

	switch(cullFaceChoice)
	{
	case 0:
		cullFaceMode = GL_BACK;
		break;
	case 1:
		cullFaceMode = GL_FRONT;
		break;
	case 2:
		cullFaceMode = GL_FRONT_AND_BACK;
		break;
	}

	switch(frontFaceChoice)
	{
	case 0:
		frontFaceMode = GL_CCW;
		break;
	case 1:
		frontFaceMode = GL_CW;
		break;
	}

	glFrontFace(frontFaceMode);
	glCullFace(cullFaceMode);

	glBegin(GL_QUADS);
	
	glColor3d(1.0f, 0.5f, 0.5f);
	glVertex2d(-0.9, -0.4);
	glVertex2d(-0.1, -0.4);
	glVertex2d(-0.1,  0.4);
	glVertex2d(-0.9,  0.4);

	glColor3f(0.5f, 1.0f, 0.5f);
	glVertex2d(0.1, -0.4);
	glVertex2d(0.1,  0.4);
	glVertex2d(0.9,  0.4);
	glVertex2d(0.9, -0.4);

	glEnd();

	glutSwapBuffers();
}


int main(int argc, char **argv)
{ 
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);  
	glutInitWindowSize( 300, 300 );

	mainWindow = glutCreateWindow( "Culling Tutor" );
	glutDisplayFunc( display );
	glutReshapeFunc( reshape );    

	GLUI *glui = GLUI_Master.create_glui( "GLUI" );

	glui->add_checkbox("Culling Enabled", &cullingEnabled);

	GLUI_Panel *glCullFace_panel = glui->add_panel("glCullFace(...)");
	GLUI_RadioGroup *glCullFace_radioGroup = glui->add_radiogroup_to_panel(glCullFace_panel, &cullFaceChoice);
	glui->add_radiobutton_to_group(glCullFace_radioGroup, "GL_BACK");
	glui->add_radiobutton_to_group(glCullFace_radioGroup, "GL_FRONT");
	glui->add_radiobutton_to_group(glCullFace_radioGroup, "GL_FRONT_AND_BACK");

	GLUI_Panel *glFrontFace_panel = glui->add_panel("glFrontFace(...)");
	glFrontFace_panel->set_alignment(GLUI_ALIGN_LEFT);
	GLUI_RadioGroup *glFrontFace_radioGroup = glui->add_radiogroup_to_panel(glFrontFace_panel, &frontFaceChoice);
	glui->add_radiobutton_to_group(glFrontFace_radioGroup, "GL_CCW");
	glui->add_radiobutton_to_group(glFrontFace_radioGroup, "GL_CW");

	glui->set_main_gfx_window( mainWindow );

	GLUI_Master.set_glutIdleFunc( idle ); 

	glutMainLoop();

	return EXIT_SUCCESS;
}