#include <windows.h>
#include <GL/glut.h>
#include <math.h>
#include <vector>
#include <stdlib.h>
#include <time.h>

using namespace std;

#define M_PI 3.14159265358979323846
#define CORNERS 256
#define INTERVAL 17

struct Ball
{
	double x, y;
	double vx, vy;
	double radius;
	double r, g, b;
	
	Ball()
	{
		x = y = 0;
		vx = 0.1; vy = 0;
		radius = 0.1;
		r = g = b = 1.0;
	}

	void setPosition(double _x, double _y)
	{
		x = _x;
		y = _y;
	}

	void setVelocity(double _vx, double _vy)
	{
		vx = _vx;
		vy = _vy;
	}

	void setRadius(double _radius)
	{
		radius = _radius;
	}

	void setColor(double _r, double _g, double _b)
	{
		r = _r;
		g = _g;
		b = _b;
	}

	void setEverything(double _x, double _y,
		double _vx, double _vy,
		double _radius,
		double _r, double _g, double _b)
	{
		setPosition(_x, _y);
		setVelocity(_vx, _vy);
		setRadius(_radius);
		setColor(_r, _g, _b);
	}

	Ball(double _x, double _y,
		double _vx, double _vy,
		double _radius,
		double _r, double _g, double _b)
	{
		setEverything(_x, _y, _vx, _vy, _radius, _r, _g, _b);
	}
};

vector<Ball> balls;

void reshape(int w, int h)
{
	glViewport(0,0,w,h);

	if (w == 0) w = 1;
	if (h == 0) h = 1;

	double aspect = (double)w / h;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (w > h)
		gluOrtho2D(-aspect, aspect, -1, 1);
	else
		gluOrtho2D(-1, 1, -1/aspect, 1/aspect);
}

void drawBall(Ball &b)
{
	glColor3d(b.r, b.g, b.b);
	glBegin(GL_TRIANGLE_FAN);
	{
		glVertex2d(b.x, b.y);
		for(int i=0;i<CORNERS;i++)
		{
			double theta = 2*M_PI*i/CORNERS;
			double x = b.x + b.radius * cos(theta);
			double y = b.y + b.radius * sin(theta);
			glVertex2d(x,y);
		}
		glVertex2d(b.x + b.radius, b.y);
	}
	glEnd();
}

void display()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	
	for(int i=0;i<(int)balls.size();i++)
		drawBall(balls[i]);

	glutSwapBuffers();
}

void updateBall(Ball &b)
{
	b.x += b.vx;
	b.y += b.vy;

	if (b.x + b.radius > 1)
	{
		b.x = 1 - b.radius;
		b.vx = -fabs(b.vx);
	}
	if (b.x - b.radius < -1)
	{
		b.x = -1 + b.radius;
		b.vx = fabs(b.vx);
	}
	if (b.y + b.radius > 1)
	{
		b.y = 1 - b.radius;
		b.vy = -fabs(b.vy);
	}
	if (b.y - b.radius < -1)
	{
		b.y = -1 + b.radius;
		b.vy = fabs(b.vy);
	}
}

void animate(int param)
{
	for(int i=0;i<(int)balls.size();i++)
		updateBall(balls[i]);

	glutTimerFunc(INTERVAL, animate, 0);
	glutPostRedisplay();
}

double random(double lower=0.0, double upper=1.0, int granularity=RAND_MAX-1)
{
	double width = (upper - lower) / granularity;
	return lower + width * (rand() % (granularity+1));
}

void newRandomBall()
{
	Ball newBall;
	newBall.setPosition(random(-1, 1, 100), random(-1, 1, 100));
	newBall.setVelocity(random(-0.05, 0.05, 100), random(-0.05, 0.05, 100));
	newBall.setRadius(random(0.02, 0.1, 100));
	newBall.setColor(random(0.5, 1.0), random(0.5, 1.0), random(0.5, 1.0));
	balls.push_back(newBall);
}

void keyboard(unsigned char key, int x, int y)
{
	if (key == ' ')
	{
		newRandomBall();
		glutPostRedisplay();
	}
	else if (key == 27)
		exit(0);
}

int main(int argc, char **argv)
{
	srand(time(NULL));

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutInitWindowSize(600,600);
	glutCreateWindow("Bouncing Balls");
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
	glutTimerFunc(0, animate, 0);
	glutKeyboardFunc(keyboard);

	glutMainLoop();
}